searchData={"items":[{"type":"module","title":"Camarero","doc":"## [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/camarero/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/camarero/workflows/Dialyzer/badge.svg) Lightweight Json API server, embeddable into any project\n\n**Camarero** is a ready-to-use solution to add some JSON API functionality to your existing application, or to implement the read-only JSON API from the scratch when more sophisticated (read: heavy) solutions are not desirable.\n\n![Camarero Ties](https://raw.githubusercontent.com/am-kantox/camarero/master/stuff/camarero.png)\n\nIt is designed to be very simple and handy for read-only web access to the data. It might be a good candidate to replace _Redis_ or any other key-value store. **It is blazingly, deadly fast**.\n\nHere are response times for the 1M key-value storage behind.\n\n![1M key-value storage lookup: 10μs±](https://raw.githubusercontent.com/am-kantox/camarero/master/stuff/1M.png)","ref":"Camarero.html"},{"type":"module","title":"Implementation details - Camarero","doc":"**Camarero** is supposed to be plugged into the functional application. It handles the configured routes/endpoints by delegating to the configured handler modules. The simplest configuration might looks like:\n\n```elixir\nconfig :camarero,\n  carta: [Camarero.Carta.Heartbeat],\n  root: \"api/v1\"\n```\n\nThe above is the default; `/api/v1` would be the root of the web server, single `Camarero.Carta.Heartbeat` module is declared as handler. The handlers might be also added dynamically by calls to `Camarero.Catering.route!`.\n\n#","ref":"Camarero.html#module-implementation-details"},{"type":"module","title":"Handlers - Camarero","doc":"_Handler_ is a module implementing `Camarero.Plato` behaviour. It consists of methods to manipulate the conteiner behind it. Any module might implement this behaviour to be used as a handler for incoming HTTP requests.\n\nThere is also `Camarero.Tapas` behaviour scaffolding the container implementation inside `Camarero.Plato`.\n\nThe default implementation using `%{}` map as a container, looks pretty simple:\n\n```elixir\ndefmodule Camarero.Carta.Heartbeat do\n  use Camarero\nend\n```\n\nThree different scaffolding implementations are currently supported with `scaffold: :impl` keyword parameter passed to `use Camarero`:\n\n- `scaffold: :full` [_default_] — the full implementation of `Camarero.Plato` is used;\n- `scaffold: :access` — `Camarero.Tapas` implementation is scaffolded only;\n- `scaffold: :none` — no scaffold is used.\n\nThis is an exact exerpt from `Heartbeat` module that comes with this package. For more complicated/sophisticated usages please refer to the [documentation](https://hexdocs.pm/camarero).\n\nAll the methods from both `Camarero.Tapas` and `Camarero.Plato` default implementations are overridable. E. g. to use the custom route for the module (default is the not fully qualified underscored module name,) as well as custom container, one might do the following:\n\n```elixir\ndefmodule Camarero.Carta.Heartbeat do\n  use Camarero, into: %MyStructWithAccessBehaviour{}\n\n  @impl Camarero.Plato\n  def plato_route(), do: \"internal/heartbeat\"\nend\n```\n\n#","ref":"Camarero.html#module-handlers"},{"type":"module","title":"Web server config - Camarero","doc":"**Camarero** runs over _Cowboy2_ with _Plug_. To configure _Cowboy_, one might specify in the `config.exs` file:\n\n```elixir\nconfig :camarero,\n  cowboy: [port: 4001, scheme: :http, options: []]\n```","ref":"Camarero.html#module-web-server-config"},{"type":"module","title":"Installation - Camarero","doc":"```elixir\ndef deps do\n  [\n    {:camarero, \"~> 1.0\"}\n  ]\nend\n```","ref":"Camarero.html#module-installation"},{"type":"module","title":"Changelog - Camarero","doc":"### `1.0.2`\n\n- pass `headers` and `cookies` to `reshape/2`\n\n### `1.0.0`\n\n- correct handling of `:value` return (not relying on map keys order)\n- no more functionality expected\n\n### `0.13`\n\n- `Plato.reshape/1` to allow reshaping of any incoming data into expected `%{\"key\" => _, \"value\" => _}`\n- make sure to use `v0.13.3` with latest `Plug` library\n\n### `0.7`\n\n- Added support for `HTTP PUT` method\n\n### `0.5`\n\n- Ability to subscribe to incoming requests with `Envío.Subscriber` (see `test/envio_test.exs` for inspiration.)\n\n### `0.4`\n\n- basic CRUD (`GET /`, `GET /:id`, `POST /`, `DELETE /:id`),\n- better support for many handlers,\n- `response_as: :value` to return raw values instead of valid JSON objects.","ref":"Camarero.html#module-changelog"},{"type":"module","title":"Is it of any good? - Camarero","doc":"Sure it is.","ref":"Camarero.html#module-is-it-of-any-good"},{"type":"module","title":"Benchmarks - Camarero","doc":"To benchmark the application one should install [`wrk`](https://github.com/wg/wrk),\nrun the application and _then_ run the `wrk.sh` script located in `wrk` folder.\n\nHere are the results it produced on my laptop.\n\n```\n=================================================================\n NB! Make sure you have a running Camarero app:\n\n      mix clean && mix run --preload-modules --no-halt\n=================================================================\n\n Performing 10 sec POSTs and 5 sec GETs afterwards.\n This will INSERT 300K key-values approx and READ 200K approx.\n\n=================================================================\n\nRunning 10s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    44.91ms   48.53ms 422.59ms   83.52%\n    Req/Sec     1.29k   212.59     3.09k    74.36%\n  311912 requests in 10.10s, 42.59MB read\nRequests/sec:  30884.14\nTransfer/sec:      4.22MB\nRunning 5s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    39.35ms   45.92ms 401.63ms   84.53%\n    Req/Sec     1.48k   375.94     4.71k    79.09%\n  179565 requests in 5.10s, 30.11MB read\nRequests/sec:  35211.34\nTransfer/sec:      5.90MB\n\n=================================================================\n\n Performing 10 sec POSTs and 5 sec DELETEs afterwards.\n This will INSERT 300K key-values approx and DELETE 200K approx.\n\n=================================================================\n\nRunning 10s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    46.32ms   52.38ms 457.89ms   84.01%\n    Req/Sec     1.31k   302.33     5.09k    79.26%\n  316401 requests in 10.10s, 43.20MB read\nRequests/sec:  31332.56\nTransfer/sec:      4.28MB\nRunning 5s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    40.17ms   46.46ms 406.37ms   84.13%\n    Req/Sec     1.49k   360.10     6.55k    84.83%\n  179722 requests in 5.10s, 38.97MB read\nRequests/sec:  35241.01\nTransfer/sec:      7.64MB\n\n=================================================================\n```\n\n## [Documentation](http://hexdocs.pm/camarero)","ref":"Camarero.html#module-benchmarks"},{"type":"module","title":"Camarero.Catering","doc":"The `DynamicSupervisor` to manage all the handlers.\n\nHandlers might be added through `config.exs` file statically _or_\n  via call to `route!/1` dynamically. The latter accepts all types of `child_spec`\n  acceptable by `DynamicSupervisor.start_child/2`.\n\nThis module is started in the application supervision tree and keeps track\n  on all the handlers.","ref":"Camarero.Catering.html"},{"type":"function","title":"Camarero.Catering.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Camarero.Catering.html#child_spec/1"},{"type":"function","title":"Camarero.Catering.route!/1","doc":"Declares and stores the new route. If the route is already set, logs an error\n  message to the log and acts as NOOP.","ref":"Camarero.Catering.html#route!/1"},{"type":"function","title":"Camarero.Catering.start_link/1","doc":"Starts the `DynamicSupervisor` _and_ `Camarero.Catering.Routes`,\n  linked to the current process.\n\nUpon start, loads `:camarero, :carta` config setting and adds routes for all\n  the statically configured handlers.","ref":"Camarero.Catering.html#start_link/1"},{"type":"behaviour","title":"Camarero.Plato","doc":"This behaviour is high-level abstraction of the container begind handlers.\n\nAll handlers are supposed to implement this behaviour. The simplest way\nis to `use Camarero.Plato` in the handler module; that will inject\nthe default boilerlate using `%{binary() => any()}` map as a container behind.\n\nDefault implementation uses `Camarero.Tapas` as low-level container implementation.","ref":"Camarero.Plato.html"},{"type":"callback","title":"Camarero.Plato.plato_all/0","doc":"Returns the container itself, as is","ref":"Camarero.Plato.html#c:plato_all/0"},{"type":"callback","title":"Camarero.Plato.plato_delete/1","doc":"Deletes the key-value pair for the key specified","ref":"Camarero.Plato.html#c:plato_delete/1"},{"type":"callback","title":"Camarero.Plato.plato_get/1","doc":"Returns the value for the key specified","ref":"Camarero.Plato.html#c:plato_get/1"},{"type":"callback","title":"Camarero.Plato.plato_put/2","doc":"Sets the value for the key specified (intended to be used from the application)","ref":"Camarero.Plato.html#c:plato_put/2"},{"type":"callback","title":"Camarero.Plato.plato_route/0","doc":"Returns the route this module is supposed to be mounted to","ref":"Camarero.Plato.html#c:plato_route/0"},{"type":"callback","title":"Camarero.Plato.reshape/1","doc":"Returns the key-value map out of a random input","ref":"Camarero.Plato.html#c:reshape/1"},{"type":"callback","title":"Camarero.Plato.reshape/2","doc":"Returns the key-value map out of a random input, the second argument contains additional request data","ref":"Camarero.Plato.html#c:reshape/2"},{"type":"type","title":"Camarero.Plato.status_code/0","doc":"HTTP status code","ref":"Camarero.Plato.html#t:status_code/0"},{"type":"behaviour","title":"Camarero.Tapas","doc":"This behaviour is high-level abstraction of the container begind handlers.\n\nAññ handlers are supposed to implement this behaviour. The simplest way\nis to `use Camarero.Plato` in the handler module; that will inject\nthe default boilerlate using `%{binary() => any()}` map as a container behind.\n\nDefault implementation uses `Camarero.Tapas` as low-level container implementation.","ref":"Camarero.Tapas.html"},{"type":"callback","title":"Camarero.Tapas.tapas_delete/2","doc":"Deletes the key-value pair for the key specified from the container","ref":"Camarero.Tapas.html#c:tapas_delete/2"},{"type":"callback","title":"Camarero.Tapas.tapas_get/2","doc":"Retrieves from the container and returns the value for the key specified","ref":"Camarero.Tapas.html#c:tapas_get/2"},{"type":"callback","title":"Camarero.Tapas.tapas_into/0","doc":"Returns the empty container to be used to store the data.\n\nThe returned term must implement `Access` behaviour since it’d be used from CR[U]D\n  operations (`c:tapas_get/2`, `c:tapas_put/3`, and `c:tapas_delete/2`).","ref":"Camarero.Tapas.html#c:tapas_into/0"},{"type":"callback","title":"Camarero.Tapas.tapas_put/3","doc":"Sets the value for the key specified (intended to be used from the application)","ref":"Camarero.Tapas.html#c:tapas_put/3"},{"type":"type","title":"Camarero.Tapas.t/0","doc":"`Camarero.Tapas` allows anything implementing `Access` behaviour as a container","ref":"Camarero.Tapas.html#t:t/0"},{"type":"module","title":"Camarero.Catering.Routes","doc":"Internal state for all the routes known to this application","ref":"Camarero.Catering.Routes.html"},{"type":"function","title":"Camarero.Catering.Routes.child_spec/1","doc":"Internally stored map of routes to handlers","ref":"Camarero.Catering.Routes.html#child_spec/1"},{"type":"function","title":"Camarero.Catering.Routes.get/1","doc":"Retrieves the handler for the route specified","ref":"Camarero.Catering.Routes.html#get/1"},{"type":"function","title":"Camarero.Catering.Routes.put/2","doc":"Stores a handler for the route specified","ref":"Camarero.Catering.Routes.html#put/2"},{"type":"function","title":"Camarero.Catering.Routes.start_link/2","doc":"Starts an agent linked to the current process","ref":"Camarero.Catering.Routes.html#start_link/2"},{"type":"function","title":"Camarero.Catering.Routes.state/0","doc":"Returns the whole mapping of routes to handlers","ref":"Camarero.Catering.Routes.html#state/0"},{"type":"type","title":"Camarero.Catering.Routes.t/0","doc":"","ref":"Camarero.Catering.Routes.html#t:t/0"},{"type":"extras","title":"Camarero","doc":"# Camarero\n\n## [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/camarero/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/camarero/workflows/Dialyzer/badge.svg) Lightweight Json API server, embeddable into any project\n\n**Camarero** is a ready-to-use solution to add some JSON API functionality to your existing application, or to implement the read-only JSON API from the scratch when more sophisticated (read: heavy) solutions are not desirable.\n\n![Camarero Ties](https://raw.githubusercontent.com/am-kantox/camarero/master/stuff/camarero.png)\n\nIt is designed to be very simple and handy for read-only web access to the data. It might be a good candidate to replace _Redis_ or any other key-value store. **It is blazingly, deadly fast**.\n\nHere are response times for the 1M key-value storage behind.\n\n![1M key-value storage lookup: 10μs±](https://raw.githubusercontent.com/am-kantox/camarero/master/stuff/1M.png)","ref":"camarero.html"},{"type":"extras","title":"Implementation details - Camarero","doc":"**Camarero** is supposed to be plugged into the functional application. It handles the configured routes/endpoints by delegating to the configured handler modules. The simplest configuration might looks like:\n\n```elixir\nconfig :camarero,\n  carta: [Camarero.Carta.Heartbeat],\n  root: \"api/v1\"\n```\n\nThe above is the default; `/api/v1` would be the root of the web server, single `Camarero.Carta.Heartbeat` module is declared as handler. The handlers might be also added dynamically by calls to `Camarero.Catering.route!`.\n\n#","ref":"camarero.html#implementation-details"},{"type":"extras","title":"Handlers - Camarero","doc":"_Handler_ is a module implementing `Camarero.Plato` behaviour. It consists of methods to manipulate the conteiner behind it. Any module might implement this behaviour to be used as a handler for incoming HTTP requests.\n\nThere is also `Camarero.Tapas` behaviour scaffolding the container implementation inside `Camarero.Plato`.\n\nThe default implementation using `%{}` map as a container, looks pretty simple:\n\n```elixir\ndefmodule Camarero.Carta.Heartbeat do\n  use Camarero\nend\n```\n\nThree different scaffolding implementations are currently supported with `scaffold: :impl` keyword parameter passed to `use Camarero`:\n\n- `scaffold: :full` [_default_] — the full implementation of `Camarero.Plato` is used;\n- `scaffold: :access` — `Camarero.Tapas` implementation is scaffolded only;\n- `scaffold: :none` — no scaffold is used.\n\nThis is an exact exerpt from `Heartbeat` module that comes with this package. For more complicated/sophisticated usages please refer to the [documentation](https://hexdocs.pm/camarero).\n\nAll the methods from both `Camarero.Tapas` and `Camarero.Plato` default implementations are overridable. E. g. to use the custom route for the module (default is the not fully qualified underscored module name,) as well as custom container, one might do the following:\n\n```elixir\ndefmodule Camarero.Carta.Heartbeat do\n  use Camarero, into: %MyStructWithAccessBehaviour{}\n\n  @impl Camarero.Plato\n  def plato_route(), do: \"internal/heartbeat\"\nend\n```\n\n#","ref":"camarero.html#handlers"},{"type":"extras","title":"Web server config - Camarero","doc":"**Camarero** runs over _Cowboy2_ with _Plug_. To configure _Cowboy_, one might specify in the `config.exs` file:\n\n```elixir\nconfig :camarero,\n  cowboy: [port: 4001, scheme: :http, options: []]\n```","ref":"camarero.html#web-server-config"},{"type":"extras","title":"Installation - Camarero","doc":"```elixir\ndef deps do\n  [\n    {:camarero, \"~> 1.0\"}\n  ]\nend\n```","ref":"camarero.html#installation"},{"type":"extras","title":"Changelog - Camarero","doc":"### `1.0.2`\n\n- pass `headers` and `cookies` to `reshape/2`\n\n### `1.0.0`\n\n- correct handling of `:value` return (not relying on map keys order)\n- no more functionality expected\n\n### `0.13`\n\n- `Plato.reshape/1` to allow reshaping of any incoming data into expected `%{\"key\" => _, \"value\" => _}`\n- make sure to use `v0.13.3` with latest `Plug` library\n\n### `0.7`\n\n- Added support for `HTTP PUT` method\n\n### `0.5`\n\n- Ability to subscribe to incoming requests with `Envío.Subscriber` (see `test/envio_test.exs` for inspiration.)\n\n### `0.4`\n\n- basic CRUD (`GET /`, `GET /:id`, `POST /`, `DELETE /:id`),\n- better support for many handlers,\n- `response_as: :value` to return raw values instead of valid JSON objects.","ref":"camarero.html#changelog"},{"type":"extras","title":"Is it of any good? - Camarero","doc":"Sure it is.","ref":"camarero.html#is-it-of-any-good"},{"type":"extras","title":"Benchmarks - Camarero","doc":"To benchmark the application one should install [`wrk`](https://github.com/wg/wrk),\nrun the application and _then_ run the `wrk.sh` script located in `wrk` folder.\n\nHere are the results it produced on my laptop.\n\n```\n=================================================================\n NB! Make sure you have a running Camarero app:\n\n      mix clean && mix run --preload-modules --no-halt\n=================================================================\n\n Performing 10 sec POSTs and 5 sec GETs afterwards.\n This will INSERT 300K key-values approx and READ 200K approx.\n\n=================================================================\n\nRunning 10s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    44.91ms   48.53ms 422.59ms   83.52%\n    Req/Sec     1.29k   212.59     3.09k    74.36%\n  311912 requests in 10.10s, 42.59MB read\nRequests/sec:  30884.14\nTransfer/sec:      4.22MB\nRunning 5s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    39.35ms   45.92ms 401.63ms   84.53%\n    Req/Sec     1.48k   375.94     4.71k    79.09%\n  179565 requests in 5.10s, 30.11MB read\nRequests/sec:  35211.34\nTransfer/sec:      5.90MB\n\n=================================================================\n\n Performing 10 sec POSTs and 5 sec DELETEs afterwards.\n This will INSERT 300K key-values approx and DELETE 200K approx.\n\n=================================================================\n\nRunning 10s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    46.32ms   52.38ms 457.89ms   84.01%\n    Req/Sec     1.31k   302.33     5.09k    79.26%\n  316401 requests in 10.10s, 43.20MB read\nRequests/sec:  31332.56\nTransfer/sec:      4.28MB\nRunning 5s test @ http://127.0.0.1:4001/api/v1/crud\n  24 threads and 1000 connections\n  Thread Stats   Avg      Stdev     Max   +/- Stdev\n    Latency    40.17ms   46.46ms 406.37ms   84.13%\n    Req/Sec     1.49k   360.10     6.55k    84.83%\n  179722 requests in 5.10s, 38.97MB read\nRequests/sec:  35241.01\nTransfer/sec:      7.64MB\n\n=================================================================\n```\n\n## [Documentation](http://hexdocs.pm/camarero)","ref":"camarero.html#benchmarks"}],"content_type":"text/markdown"}